@@ -13,6 +13,7 @@ import pandas as pd
 import inspect
 from inspect import signature
 from urllib.parse import urlparse
+import ast
 
 logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                     level=logging.INFO)
@@ -405,9 +406,12 @@ class DataPlate:
             if 'df' in kwargs:
                 # df = pd.read_json(args['df'])
                 kwargs['df'] = kwargs['df'].to_json()
-                json_kwargs_object = json.dumps(kwargs)
-            else:
-                return wr.s3.to_csv(*sba.args, **sba.kwargs)
+                # json_kwargs_object = json.dumps(kwargs)
+                if len(kwargs['df']) <= 5:
+                    logging.error('Empty dataframe !')
+                    return
+            # else:
+            return wr.s3.to_csv(*sba.args, **sba.kwargs)
 
 
             headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
@@ -467,9 +471,12 @@ class DataPlate:
             if 'df' in kwargs:
                 # df = pd.read_json(args['df'])
                 kwargs['df'] = kwargs['df'].to_json()
-                json_kwargs_object = json.dumps(kwargs)
-            else:
-                return wr.s3.to_json(*sba.args, **sba.kwargs)
+                # json_kwargs_object = json.dumps(kwargs)
+                if len(kwargs['df']) <= 5:
+                    logging.error('Empty dataframe !')
+                    return
+            # else:
+            return wr.s3.to_json(*sba.args, **sba.kwargs)
 
 
             headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
@@ -529,9 +536,12 @@ class DataPlate:
             if 'df' in kwargs:
                 # df = pd.read_json(args['df'])
                 kwargs['df'] = kwargs['df'].to_json()
-                json_kwargs_object = json.dumps(kwargs)
-            else:
-                return wr.s3.to_json(*sba.args, **sba.kwargs)
+                # json_kwargs_object = json.dumps(kwargs)
+                if len(kwargs['df']) <= 5:
+                    logging.error('Empty dataframe !')
+                    return
+            # else:
+            return wr.s3.to_json(*sba.args, **sba.kwargs)
 
 
             headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
@@ -578,6 +588,7 @@ class DataPlate:
     write_to_s3_parquet.__doc__ = wr.s3.to_parquet.__doc__.replace('awswrangler','dataplate')
 
 
+
     def run_notebook(self, notebook_file_path, instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
         """Run a notebook in SageMaker Processing producing a new output notebook.
         Args:
@@ -646,7 +657,7 @@ class DataPlate:
                 if r.text:
                     rJson = json.loads(r.text)
                     logging.info('run notebook finished successfully.')
-                    return json.dumps(rJson)
+                    return r.text.replace('"','')#json.dumps(rJson)
                 else:
                     logging.exception('Could not find proper output, please check your parameters')
                 # return r.text
@@ -986,7 +997,7 @@ class DataPlate:
 
 
 
-    def describe_notebook_run(self, job_name= None):
+    def describe_notebook_run(self, job_name= None, _showprocesslogging=True):
         """Stop the named processing job.
         Args:
         job_name (string): The name of the job to stop. use list_runs_notebook to get specific notebook Job name
@@ -1021,7 +1032,8 @@ class DataPlate:
             headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
 
             try:
-                logging.info('sending request...')
+                if _showprocesslogging:
+                    logging.info('sending request...')
                 r = self.session.post( \
                     '{}/api/aws/describeNotebookRun'.format(self.base_url), params=params, data="",
                     headers=headers, stream=True, allow_redirects=False)
@@ -1032,19 +1044,22 @@ class DataPlate:
                             'Bad Access Key! Get your access key at: {}'.format(
                                 self.base_url))
                     if r.status_code == 206:
-                        logging.info('analysing notebook details...')
+                        if _showprocesslogging:
+                            logging.info('analysing notebook details...')
                         time.sleep(5)
                     raise Error(
                         'Bad HTTP exit status returned from the API: {}. Error was: {}'.
                             format(r.status_code, r.text))
 
-                logging.info('Parsing response')
+                if _showprocesslogging:
+                    logging.info('Parsing response')
                 # logging.info(str(r.text))
                 if r.text:
                     try:
                         rJson = json.loads(r.text)
                         # res = ast.literal_eval(r.text)
-                        logging.info('describe notebook run finished successfully.')
+                        if _showprocesslogging:
+                            logging.info('describe notebook run finished successfully.')
                         return rJson
                     except Exception as e:
                         logging.info('Invalid notebook details found.')
@@ -1081,7 +1096,7 @@ class DataPlate:
         while not done:
             if progress:
                 print(".", end="")
-            desc = self.describe_notebook_run(job_name=job_name)
+            desc = self.describe_notebook_run(job_name=job_name,_showprocesslogging=False)
             status = desc["Status"]
             if status != "InProgress":
                 done = True
@@ -1099,7 +1114,6 @@ class DataPlate:
         """
 
         try:
-
             params = {}
             if rule_name:
                 params['rule_name'] = rule_name
@@ -1146,7 +1160,7 @@ class DataPlate:
 
 
 
-    def schedule_notebook(self, notebook_file_path, rule_name = None, schedule = None, instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
+    def schedule_notebook(self, notebook_file_path, rule_name = None, schedule = None, event_pattern = None, instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
         """Create a schedule for running a notebook in SageMaker Processing.
 
             Creates a CloudWatch Event rule to invoke the installed Lambda either on the provided schedule or in response
@@ -1164,6 +1178,9 @@ class DataPlate:
                 schedule (str): A schedule string which defines when the job should be run. For details, 
                                 see https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/ScheduledEvents.html 
                                 (default: None)
+                event_pattern (str): A pattern for events that will trigger notebook execution. For details,
+                             see https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/CloudWatchEventsandEventPatterns.html.
+                             (default: None. Note: one of `schedule` or `event_pattern` must be specified).
                 input_path (str): The S3 object containing the notebook. If this is None, the `notebook` argument is
                                   taken as a local file to upload (default: None).
                 parameters (dict): The dictionary of parameters to pass to the notebook (default: {}).
@@ -1200,10 +1217,19 @@ class DataPlate:
             params['notebook_name'] = notebook_name if notebook_name else ""
             if not rule_name:
                 raise Error(f'rule_name is required')
-            if not schedule:
-                raise Error(f'schedule is required, see https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-create-rule-schedule.html#eb-cron-expressions')
+            if not schedule and not event_pattern:
+                raise Error(f'schedule or event_pattern is required, for cron scheduling see https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-create-rule-schedule.html#eb-cron-expressions')
+            elif schedule and len(schedule) < 4:
+                raise Error(
+                    f'schedule not defined properly, see https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-create-rule-schedule.html#eb-cron-expressions')
+            elif event_pattern and len(event_pattern) < 4:
+                raise Error(
+                    f'event_pattern not defined properly, see https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/CloudWatchEventsandEventPatterns.html')
             params['rule_name'] = rule_name
-            params['schedule'] = schedule
+            if schedule:
+                params['schedule'] = schedule
+            if event_pattern:
+                params['event_pattern'] = event_pattern
             if role:
                 params['role'] = role
 
@@ -1244,4 +1270,61 @@ class DataPlate:
 
         except Exception as e:
             logging.exception('schedule_notebook , ' + str(e))
+            raise Error(e)
+
+
+
+    def analyzeNotebookResult(self, job_name= None):
+        """Analyze the named job.
+        Args:
+        job_name (string): The name of the job to stop. use list_runs_notebook to get specific notebook Job name
+        """
+        try:
+            params = {}
+            if job_name:
+                params['jobname'] = job_name
+            else:
+                raise Error('Not a valid job name, use list_runs_notebook function to get specific notebook Job name.')
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/sec/analyzeNotebookResult'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook stops...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    # logging.info(r.text)
+                    # rJson = json.loads(r.text)
+                    logging.info('analyze notebook finished successfully.')
+                    # parsed_json = ast.literal_eval(rJson)
+                    df_compromise = pd.read_json(r.text)
+                    return df_compromise
+                    # return
+                else:
+                    logging.exception('Could not find proper result, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('stop_run_notebook , ' + str(e))
             raise Error(e)
\ No newline at end of file
